/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */
#ifndef ETHERBOOT_BITS_BYTESWAP_H
#define ETHERBOOT_BITS_BYTESWAP_H

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

static inline __attribute__ ((always_inline, const)) uint16_t
__bswap_variable_16(uint16_t x)
{
	__asm__("xchgb %b0,%h0\n\t"
		: "=q" (x)
		: "0" (x));
	return x;
}

static inline __attribute__ (( always_inline )) void
__bswap_16s ( uint16_t *x ) {
	__asm__ ( "rorw $8, %0" : "+m" ( *x ) );
}

static inline __attribute__ ((always_inline, const)) uint32_t
__bswap_variable_32(uint32_t x)
{
	__asm__("xchgb %b0,%h0\n\t"
		"rorl $16,%0\n\t"
		"xchgb %b0,%h0"
		: "=q" (x)
		: "0" (x));
	return x;
}

static inline __attribute__ (( always_inline )) void
__bswap_32s ( uint32_t *x ) {
	__asm__ ( "bswapl %0" : "=r" ( *x ) : "0" ( *x ) );
}

static inline __attribute__ ((always_inline, const)) uint64_t
__bswap_variable_64(uint64_t x)
{
	union {
		uint64_t qword;
		uint32_t dword[2]; 
	} u;

	u.qword = x;
	u.dword[0] = __bswap_variable_32(u.dword[0]);
	u.dword[1] = __bswap_variable_32(u.dword[1]);
	__asm__("xchgl %0,%1"
		: "=r" ( u.dword[0] ), "=r" ( u.dword[1] )
		: "0" ( u.dword[0] ), "1" ( u.dword[1] ) );
	return u.qword;
}

#endif /* ETHERBOOT_BITS_BYTESWAP_H */
