/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * Driver for Solarflare network controllers and boards
 * Copyright 2019 Solarflare Communications Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation, incorporated herein by reference.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

/* This is a copy of v5/src/lib/sfgpxe/dyncfg_accessors.h.
 * When possible, changes should be done in the v5 repo and backported here.
 *
 * Differences with the original file should be noted with a 'DIFF' marker, and
 * a comment explaining why the difference is kept local.
 */

/* DIFF: Include a compatibility header file. */
#include "dyncfg_sfgpxe_compat.h"

#ifndef _SFC_EF10_DYNCFG_ACCESSORS_H
#define _SFC_EF10_DYNCFG_ACCESSORS_H

#define DYNCFG_SETTING_DEFAULT_MSIX_LIMIT 32
#define DYNCFG_SETTING_DEFAULT_VF_MSIX_LIMIT 8
#define DYNCFG_SETTING_DEFAULT_VF_COUNT 0
#define DYNCFG_SETTING_DEFAULT_FIRMWARE_VARIANT -1
#define DYNCFG_SETTING_DEFAULT_PF_COUNT 1
#define DYNCFG_SETTING_DEFAULT_PORT_MODE -1
#define DYNCFG_SETTING_DEFAULT_PRIVILEGE_MASK 0
#define DYNCFG_SETTING_DEFAULT_PFIOV 0
#define DYNCFG_SETTING_DEFAULT_VLAN_ENABLED 0
#define DYNCFG_SETTING_DEFAULT_VLAN_TAG 0xffffffff
#define MSIX_LIMIT_PER_ADAPTER 2048
#define MAX_PF_COUNT 16
#define MAX_VI_MEMORY_PER_PORT_LOG2 0x0017  //Max VI count per port * Size of VI
#define GENEROSITY_FACTOR_LOG2 0x0003

#define DECLARE_CONFIG_ACCESSORS(opt_name, val_type)                           \
void gpxe_dyncfg_get_##opt_name(                                               \
    unsigned char *dyncfg_buf, int dyncfg_buf_size,                            \
    struct sfgpxe_dev_info *dev_info __maybe_unused,                           \
    val_type *res, val_type default_val);                                      \
int gpxe_dyncfg_set_##opt_name(                                                \
    unsigned char *dyncfg_buf, int dyncfg_buf_size,                            \
    struct sfgpxe_dev_info *dev_info __maybe_unused,                           \
    val_type val);

unsigned int get_port_count(uint32_t port_mode);
int update_per_pf_pcie_config(unsigned char *dyncfg_buf, int dyncfg_buf_size,
			      uint32_t pci_device_id, uint32_t port_mode);
int update_vf_aperture(unsigned char *dyncfg_buf, int dyncfg_buf_size,
		       struct sfgpxe_dev_info *dev_info, uint32_t port_mode);

DECLARE_CONFIG_ACCESSORS(msix_limit, unsigned short)
/* DIFF: Modified from 'unsigned char' to 'unsigned short'.
 * We are not constrained here in the same way as the sfgpxe code is. We know
 * these accessors will only be used directly for ef10 devices, so we can use
 * the appropriate type.
 */
// TODO: Maybe we need to stick to unsigned char for gpxe as well. cf
// hunt_dyn_cfg_fetch()
DECLARE_CONFIG_ACCESSORS(vf_msix_limit, unsigned short)
DECLARE_CONFIG_ACCESSORS(vf_count, unsigned char)
DECLARE_CONFIG_ACCESSORS(firmware_variant, unsigned)
DECLARE_CONFIG_ACCESSORS(pf_count, int16_t)
DECLARE_CONFIG_ACCESSORS(port_mode, unsigned)
DECLARE_CONFIG_ACCESSORS(privilege_mask, unsigned)
DECLARE_CONFIG_ACCESSORS(pfiov, uint8_t)
DECLARE_CONFIG_ACCESSORS(vlan_tag, uint32_t)
DECLARE_CONFIG_ACCESSORS(vswitch_type, uint32_t)
DECLARE_CONFIG_ACCESSORS(vf_aper, uint16_t)
DECLARE_CONFIG_ACCESSORS(ocsd_mode, uint32_t)

#endif
