/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */
#ifndef CURSOR_H
#define CURSOR_H

/** @file
 *
 * MuCurses cursor implementation specific header file
 *
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

struct cursor_pos {
	unsigned int y, x;
};

/**
 * Restore cursor position from encoded backup variable
 *
 * @v *win	window on which to operate
 * @v *pos	pointer to struct in which original cursor position is stored
 */
static inline void _restore_curs_pos ( WINDOW *win, struct cursor_pos *pos ) {
	wmove ( win, pos->y, pos->x );
}

/**
 * Store cursor position for later restoration
 *
 * @v *win	window on which to operate
 * @v *pos	pointer to struct in which to store cursor position
 */
static inline void _store_curs_pos ( WINDOW *win, struct cursor_pos *pos ) {
	pos->y = win->curs_y;
	pos->x = win->curs_x;
}

#endif /* CURSOR_H */
