/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

/** @file
 *
 * gcc sometimes likes to insert implicit calls to memcpy().
 * Unfortunately, there doesn't seem to be any way to prevent it from
 * doing this, or to force it to use the optimised memcpy() as seen by
 * C code; it insists on inserting a symbol reference to "memcpy".  We
 * therefore include wrapper functions just to keep gcc happy.
 *
 */

#include <string.h>

void * gcc_implicit_memcpy ( void *dest, const void *src,
			     size_t len ) asm ( "memcpy" );

void * gcc_implicit_memcpy ( void *dest, const void *src, size_t len ) {
	return memcpy ( dest, src, len );
}
